# ----------------------------------------------------------------------------
# EBook         sms
# ----------------------------------------------------------------------------
#   Macro        SMScorrfood
# ----------------------------------------------------------------------------
#   Description  SMScorrfood performs a correspondence analysis for the French food 
#                data ("food.dat"), shows the eigenvalues of the singular value
#                decomposition of the chi-matrix and displays graphical its
#                factorial decomposition. 
# ----------------------------------------------------------------------------
# Author: Lucia Jaresova
# ----------------------------------------------------------------------------

library(MASS)		

data(food)

x=food
x		        # data
a = apply(x,1,sum);a	# row frequencies		
b = apply(x,2,sum);b	# column frequencies

# estimated expected value in (i,j)-th category
# under the assumption of independence
e = a%*%t(b)/sum(a)           
round(e,0)

# chi-matrix (contributors to the chi-statistics), 
# which may be viewed as a measure of the departure 
# of the observed x_{ij} from independence
C = (x-e)/sqrt(e) 

decomp = svd(C)			# singular value decomposition
d = decomp$d;d
u = decomp$u;u
v = decomp$v;v
C = u%*%diag(d)%*%t(v)		# this should be matrix C
# eigenvalues & percentages
cbind(d^2, cumsum(d^2/sum((d^2))))


# way how to compute correspondece analysis in R

# corresp ... find the principal canonical correlation and corresponding
# 	row- and column-scores from a correspondence analysis of a two-way 
# 	contingency table 
# x ... various forms of the principal argument for specifying a two-way frequency table
# nf ... the number of factors to be computed (we need two for biplot)
#	the first two singular vectors for a sort of biplot.

co = corresp(x, nf = 7)    
names(co)
co$cor		
co$rscore       # row-scores
co$cscore	# column-scores
co$Freq		# two-way frequency table (we have it already as input)
biplot(co) 		



