## SMSdisfbank2
## Fisher's LDA and PC projection for Swiss bank notes

library(stats)
library(MASS)

data(bank2)
truth=factor(rep(c("Genuine","Forged"),each=100))

# centering
x=t(t(as.matrix(bank2))-mean(bank2))
# sphering
t=eigen(var(x))
v12=t$vectors%*%diag(1/sqrt(t$values))%*%t(t$vectors)
x=x%*%v12

fisher=as.matrix(x)%*%(lda(truth~x)$scaling)
fisher=(fisher-mean(fisher))/sd(fisher)
principal=prcomp(x)$x[,1]
principal=principal/sd(principal)

library(KernSmooth)
est.p <- bkde(principal, bandwidth=0.25)
est.f <- bkde(fisher, bandwidth=0.25)

dum=rbind(c(min(c(est.p$x,est.f$x)),-0.2),c(max(c(est.p$x,est.f$x)),max(c(est.p$y,est.f$y))))

plot(dum,type="n",main="Fisher's LDA and PC projection",xlab="",ylab="densities of projections")
lines(est.p,col="blue")
lines(est.f,col="red",lty=2)

t=(runif(200)*0.09)-0.095
points(t~fisher,col="red",pch=5,cex=0.7)
t=(runif(200)*0.09)-0.195
points(t~principal,col="blue",cex=0.7)
